# coding: utf-8
import xml.dom.minidom
import re
from copy import deepcopy

"""
xml读取工具
author:CallMeE
date:2018-06-01
"""
# 定义操作符
marks = {
    'eq': '==',
    'lt': '<',
    'gt': '>',
    'le': '<=',
    'ge': '>=',
    'not': '!=',
    'has': 'HAS'
}


class Mapper:
    _data = {}

    def sortTags(self, file: str, tree, name):
        tags = tree.getElementsByTagName(name)
        data = self._data
        file_list = file.split('/')
        file_name = re.sub('\\..*', '', file_list[len(file_list) - 1])
        if file_name not in data:
            data[file_name] = {}
        file_cache = data[file_name]
        for tag in tags:
            val = []
            for node in tag.childNodes:
                if isinstance(node, xml.dom.minidom.Text):
                    if re.sub('\s', '', node.data):
                        val.append(re.sub('\n', '', node.data))
                else:
                    val.append(id(node))
            attr = tag.getAttribute("id")
            id_bind_obj = {}
            id_bind_obj['main_sql'] = val
            # 获取条件标签
            if_tags = self.sortIfTags(tag)
            id_bind_obj['if'] = if_tags
            for_tags = self.sortForTag(tag)
            id_bind_obj['for'] = for_tags
            file_cache[attr] = id_bind_obj

    def openDom(self, file):
        # 使用minidom解析器打开 XML 文档
        DOMTree = xml.dom.minidom.parse(file)
        tags = DOMTree.documentElement
        # 取出标签(增删查改)
        self.sortTags(file, DOMTree, 'select')
        self.sortTags(file, DOMTree, 'delete')
        self.sortTags(file, DOMTree, 'insert')
        self.sortTags(file, DOMTree, 'update')

    def getTree(self):
        return self._data

    def sortIfTags(self, tag):
        if_tags = tag.getElementsByTagName('if')
        tags_structer = {}
        for if_tag in if_tags:
            val = if_tag.childNodes[0].data
            attrs = if_tag.attributes
            attr_keys = attrs.keys()
            tag_struct = {}
            if attr_keys:
                if_key_index = 0
                for k in attr_keys:
                    if_key_struct = {}
                    attr_value = str(if_tag.getAttribute(k)).split('|')
                    attr_operator = marks.get(attr_value[0])
                    attr_operator_value = attr_value[1] if len(attr_value) > 1 else ''
                    if attr_value:
                        if_key_struct['_key'] = k
                        if_key_struct['operator'] = attr_operator
                        if_key_struct['_value'] = attr_operator_value
                        if_key_struct['add_sql'] = val
                        tag_struct[if_key_index] = if_key_struct
                        if_key_index += 1
                tags_structer[id(if_tag)] = tag_struct
        return tags_structer

    def sortForTag(self, tag):
        """
        处理循环标签(暂定使用文本元素替换标签元素)
        :param tag:
        :return:
        """
        for_tags = tag.getElementsByTagName('for')
        tags_structer = {}
        if for_tags:
            for for_tag in for_tags:
                # 获取初始化相关参数
                left_area_tag = for_tag.getAttribute('left') if for_tag.hasAttribute('left') else '('
                right_area_tag = for_tag.getAttribute('right') if for_tag.hasAttribute('right') else ')'
                sep = for_tag.getAttribute('sep') if for_tag.hasAttribute('sep') else ','

                tags_structer[id(for_tag)] = {
                    'sep': sep,
                    'left_area_tag': left_area_tag,
                    'right_area_tag': right_area_tag,
                    'arg_key': re.sub('\s*', '', re.sub('#{|}', '', for_tag.childNodes[0].data))
                }
                # new = xml.dom.minidom.Text().replaceWholeText('111')  # 创建文本节点
        return tags_structer

    def get_executable_sql(self, sql_datas: dict, method_name: str, args=None) -> str:
        """
        获取实际执行语句
        :param method_name:
        :param args:
        :return:
        """
        result_sql = deepcopy(sql_datas)[method_name]
        if isinstance(result_sql, str):
            return result_sql
        if isinstance(result_sql, dict) and args and isinstance(args, dict):
            # 如果存在参数
            # 处理条件标签
            self.check_if(result_sql, args)
            # print(result_sql)
            self.check_for(result_sql, args)
            # print(result_sql)
            return ''.join(result_sql['main_sql'])
        return ''.join(result_sql['main_sql'])

    def check_for(self, main_struct: dict, args: dict):
        """
        检查循环结构
        :param main_struct:
        :param args:
        :return:
        """
        for_struct = main_struct['for']
        if for_struct:
            # print(for_struct)
            main_struct_index = 0
            for main_struct_item in main_struct['main_sql']:
                if not isinstance(main_struct_item,str) and main_struct_item in for_struct:
                    # 遍历语句节点
                    if not isinstance(main_struct_item, str):
                        for_v = for_struct[main_struct_item]
                        arg_sequence = args.get(for_v['arg_key'], None)
                        if arg_sequence:
                            # 处理序列参数
                            index = 0
                            for i in arg_sequence:
                                if not isinstance(i, str):
                                    arg_sequence[index] = str(i)
                                index += 1
                            main_struct['main_sql'][main_struct_index] = for_v['left_area_tag'] + \
                                                                         for_v['sep'].join(arg_sequence) + \
                                                                         for_v['right_area_tag']
                        else:
                            raise KeyError('找不到对应序列:' + for_v['arg_key'])
                main_struct_index += 1

    def check_if(self, main_struct: dict, args: dict):
        """
        检查条件结构
        :param if_struct:
        :param args:
        :return:
        """
        if_struct = main_struct['if']
        if if_struct:
            main_struct_index = 0
            for main_struct_item in main_struct['main_sql']:
                if not isinstance(main_struct_item, str) and main_struct_item in if_struct:
                    # 存在条件结构数据
                    compile_result = True
                    for if_v in if_struct[main_struct_item].values():
                        # 遍历每个if标签
                        # 根据操作符进行判断
                        compile_key = if_v['_key']
                        if if_v['operator'] == marks['has']:
                            if compile_key in args:
                                # print(args.get(compile_key))
                                compile_result = compile_result and args.get(compile_key)
                            else:
                                compile_result = False
                        else:
                            # dict类型(标签属性结构)
                            if compile_key in args:
                                arg_value = args[compile_key]
                                if isinstance(arg_value, str):
                                    compile_result = (if_v['_value'] == arg_value) and compile_result
                                else:
                                    compile_result = eval(
                                        if_v['_value'] +
                                        if_v['operator'] +
                                        str(arg_value)
                                    ) and compile_result
                        if compile_result:
                            # 判断标签全属性条件是否通过
                            # 执行表达式结果
                            main_struct['main_sql'][main_struct_index] = if_v['add_sql']
                        else:
                            main_struct['main_sql'][main_struct_index] = ''
                main_struct_index += 1



from dophon_db import properties
from .core import curObj
from dophon_db import Pool, Connection
from dophon_db.mysql.remote.Cell import Cell
import re
from dophon_logger import *

get_logger(DOPHON).inject_logger(globals())

obj_manager = {}

# 定义项目路径
project_path = properties.project_root

cluster_manager = {}


def get_manager():
    return cluster_manager


def get_db(path, debug: bool = False, auto_fix: bool = False):
    """

    :param path:
    :param debug:
    :param auto_fix:
    :return:
    """
    x = lambda p: p.split('::')
    if isinstance(path, str):
        from . import mode_assert
        # 切分出别名和路径
        cluster_path = mode_assert('CLUSTER')(x)(path)
        assert len(cluster_path) > 1, '无法解释的路径格式'
        if auto_fix:
            pattern = re.sub(r'\\', r'\\\\', re.sub('/', '\\/', project_path))
            if not re.search(pattern, cluster_path[1]):
                r_path = str(project_path) + str(cluster_path[1])
            else:
                r_path = str(cluster_path[1])
        else:
            r_path = cluster_path[1]
        # 返回预设连接池所生成的数据库对象
        alias_name = cluster_path[0]
        if alias_name in cluster_manager:
            return curObj(r_path, True, debug, db=cluster_manager[alias_name])
        else:
            raise KeyError('别名不存在')
    elif isinstance(path, Cell):
        logger.error('暂不支持远程映射集')
    else:
        raise TypeError('结果集路径类型错误')


def init_cluster_manager():
    if hasattr(properties, 'db_cluster'):
        clusters = properties.db_cluster
        # 初始化分片连接池
        logger.info('初始化分片连接池')
        for cluster in clusters:
            pool = Pool.Pool()
            pool.initPool(properties.pool_conn_num, Connection.Connection, conn_kwargs=cluster)
            # print(f'{cluster["alias"]}--{pool.cache_conn_kwargs}')
            # print(id(pool))
            cluster_manager[cluster['alias']] = pool
        logger.info('分片连接池初始化完毕')



# coding: utf-8
import re

from dophon_logger import *

logger = get_logger(DOPHON)

"""
sql分页工具(自带正则寻值)

(不区分大小写)
*num* ----->页码
*size* ------>页容

待实现:
1.参照mybatis的pagehelper


author:CallMeE
date:2018-06-01
"""

logger.inject_logger(globals())


# 打包分页信息
def pkg_page_info(page_num=1, page_size=1, page_model=[]):
    if not page_model:
        return {'page_num': locals()['page_num'], 'page_size': locals()['page_size']}
    else:
        page_info = {}
        # 选取对应key写入数据
        for key in page_model:
            if re.match('.*[nN][uU][mM].*', string=key):
                page_info[key] = page_num
            if re.match('.*[sS][iI][zZ][eE].*', string=key):
                page_info[key] = page_size
        return page_info


# 解包分页信息
def depkg_page_info(page_info: dict):
    for key in page_info:
        if re.match('.*[nN][uU][mM].*', string=key):
            _page_num = page_info[key]
        if re.match('.*[sS][iI][zZ][eE].*', string=key):
            _page_size = page_info[key]
    return 'limit ' + str(
        0 if (int(_page_num) - 1) * int(_page_size) < 0 else (int(_page_num) - 1) * int(_page_size)) + ',' + str(
        _page_size)


def fix_page_info(page_info: dict):
    """
    修正分页信息
    :param page_info:
    :return:
    """
    for key in page_info:
        if re.match('.*[nN][uU][mM].*', string=key):
            _page_num = page_info[key]
        if re.match('.*[sS][iI][zZ][eE].*', string=key):
            _page_size = page_info[key]
    return {'page_num': _page_num, 'page_size': _page_size}


# 分页装饰器
def pkg_pageobj(f):
    """
    封装结果集为分页信息结果集 -> Pages
    :param f:
    :return:
    """
    # print(f)

    def f_args(*args, **kwargs):
        # 执行查询
        result = f(*args, **kwargs)
        # 结果校验
        if isinstance(result, list):
            # 分页信息检查
            if 'pageInfo' in kwargs and kwargs['pageInfo']:
                page_obj = fix_page_info(kwargs['pageInfo'])
                page_obj['list'] = result
                return page_obj
            else:
                logger.error('不存在分页信息')
                return result
        else:
            logger.error('结果不符合分页策略')

    return f_args



from dophon_db import Connection
from dophon_db import Pool
from dophon_db import utils
import types
from dophon_db.orm.db_obj.type_dict import db_type_python_dict
from dophon_db.orm.db_obj.type_dict import set_check
from dophon_db.orm.db_obj.function_class import *
from dophon_db.orm.query_structor import Struct

from dophon_logger import *

logger = get_logger(DOPHON)

logger.inject_logger(globals())

# 初始化表结构缓存
table_cache = {}


def create_class(table_name: str, table_args: list):
    """
    创建数据表类
    :param table_name:  表名
    :param table_args: 表参数
    :return:
    """
    class_obj = type(table_name, (
        SetAble,
        JoinAble,
        ValueAble,
        Struct,
        Parseable,
        Flushable,
        Pageable),
                     {'__alias': table_name, 'table_map_key': table_name})
    default_arg_list = []
    for table_arg in table_args:
        # 获取表字段名以及属性
        table_arg_field = table_arg['Field']
        table_arg_type = table_arg['Type']
        table_arg_null = table_arg['Null']
        table_arg_key = table_arg['Key']
        table_arg_default = table_arg['Default']

        '''
        映射类属性方法组装
        '''
        setter_code = compile(
            'def setter_' + table_arg_field + '(self,value):' +
            '\n\tself._' + table_arg_field + ' = value' +
            '\n\tself.append(\'' + table_arg_field + '\')',
            '',
            'exec'
        )
        setter_function_code = [c for c in setter_code.co_consts if isinstance(c, types.CodeType)][0]
        setter_method = set_check(table_arg_type)(types.FunctionType(setter_function_code, {}))

        getter_code = compile(
            'def getter_' + table_arg_field + '(self):' +
            '\n\treturn self._' + table_arg_field,
            '',
            'exec'
        )
        getter_function_code = [c for c in getter_code.co_consts if isinstance(c, types.CodeType)][0]
        getter_method = types.FunctionType(getter_function_code, {})

        setattr(
            class_obj,
            '_' + table_arg_field,
            table_arg_default if table_arg_null == 'YES' else None,
        )

        setattr(
            class_obj,
            table_arg_field,
            property(getter_method, setter_method)
        )

        default_arg_list.append(table_arg_field)

    # 设定默认字段列表(所有字段)
    setattr(class_obj, '__default_arg_list', default_arg_list)

    '''
    映射类固定方法组装
    '''
    # 重载直接调用运算符
    callable_code = compile(
        'def __call__(self,call_list):' +
        '\n\treturn self.get_fields(call_list)',
        '',
        'exec'
    )
    callable_function_code = [c for c in callable_code.co_consts if isinstance(c, types.CodeType)][0]
    callable_method = types.FunctionType(callable_function_code, {})

    setattr(
        class_obj,
        '__call__',
        callable_method
    )

    # 重载映射类别名运算符
    alias_code = compile(
        'def alias(self,alias_name:str):' +
        '\n\tself.__alias=alias_name' +
        '\n\treturn self',
        '',
        'exec'
    )
    alias_function_code = [c for c in alias_code.co_consts if isinstance(c, types.CodeType)][0]
    alias_method = types.FunctionType(alias_function_code, {})

    setattr(
        class_obj,
        'alias',
        alias_method
    )

    return class_obj


class OrmManager:
    __table_cache = {}

    def add_orm_obj(self, table_obj: object):
        if 'table_name' in table_obj:
            # 添加表名单位
            table_name = table_obj['table_name']
            table_alias = table_obj['table_alias'] if table_obj['table_alias'] else table_obj['table_name']
            # 编译表名属性方法(property)
            getter_module_code = compile(
                'def ' + table_obj['table_name'] + '(self):\n\treturn self._' + table_obj['table_name'],
                '',
                'exec'
            )
            function_code = [c for c in getter_module_code.co_consts if isinstance(c, types.CodeType)][0]
            getter_method = types.FunctionType(function_code, {})
            # 编译获取管理器连接池方法(property)
            pool_getter_module_code = compile(
                'def pool_getter(self):\n\treturn self.connection_pool',
                '',
                'exec'
            )
            pool_getter_function_code = [c for c in pool_getter_module_code.co_consts if isinstance(c, types.CodeType)][0]
            pool_getter_method = types.FunctionType(pool_getter_function_code, {})
            # 获取表结构
            table_arg = table_obj['table_obj']
            if not search_class_by_name(table_alias):
                # 组装新类
                table_class = create_class(table_name, table_arg)
                save_cache(table_alias, table_class)
            else:
                table_class = get_cache(table_alias)
            # 植入类内
            setattr(OrmManager, '_' + table_name, table_class)
            setattr(OrmManager, table_name, property(getter_method))
            # 植入类内
            setattr(table_class, 'pool_getter', pool_getter_method)
            setattr(table_class, 'connection_pool', self.connection_pool)
            self.__table_cache[table_obj['table_name']] = True
        else:
            logger.error('插入对象异常')
            raise Exception('插入对象异常')

    def has_table(self, table_name: str):
        return table_name in self.__table_cache


def init_pool_in_manager(manager, conn_kwargs):
    """
    初始化对象管理器连接池
    :param manager:
    :param conn_kwargs:
    :return:
    """
    # 组装连接池属相相关
    setattr(manager,'_connection_pool', Pool.get_pool(conn_kwargs))
    # 编译表名属性方法(property)
    getter_module_code = compile(
        'def connection_pool(self):\n\treturn self._connection_pool',
        '',
        'exec'
    )
    function_code = [c for c in getter_module_code.co_consts if isinstance(c, types.CodeType)][0]
    getter_method = types.FunctionType(function_code, {})

    # 植入类内
    setattr(OrmManager, 'connection_pool', property(getter_method))


def init_tables_in_db(manager: OrmManager, tables: list = [], conn_kwargs: dict = {}):
    logger.info('数据库全表ORM初始化开始' if not tables else '数据表' + str(tables[:]) + 'ORM初始化开始')
    connect = Connection.Connection(**conn_kwargs).getConnect()
    cursor = connect.cursor()
    cursor.execute('SHOW TABLES')
    connect.commit()
    # 整理数据表名列表
    for tup_item in cursor.fetchall():
        tup_item_name = tup_item[0]
        if '__host' in conn_kwargs and '__port' in conn_kwargs and '__database' in conn_kwargs:
            tup_item_alias = '-'.join(
                [conn_kwargs.get('__host'), str(conn_kwargs.get('__port')), conn_kwargs.get('__database'), tup_item[0]])
        else:
            tup_item_alias = tup_item_name
        if tables:
            if tup_item_name in tables:
                init_table_param(tup_item_name, manager, table_alias=tup_item_alias, conn_kwargs=conn_kwargs)
        else:
            init_table_param(tup_item_name, manager, table_alias=tup_item_alias, conn_kwargs=conn_kwargs)
    connect.close()
    logger.info('数据库ORM初始化完毕')


def init_table_param(table_name, manager: OrmManager, table_alias: str = '', conn_kwargs: dict = {}):
    connect = Connection.Connection(**conn_kwargs).getConnect()
    cursor = connect.cursor()
    cursor.execute('DESC ' + table_name)
    connect.commit()
    titles = cursor.description
    values = cursor.fetchall()
    result = utils.sort_result(values, titles, [])
    table_obj = {
        'table_alias': table_alias,
        'table_name': table_name,
        'table_obj': result
    }
    manager.add_orm_obj(table_obj)
    connect.close()


def save_cache(table_name: str, table_class: object):
    """
    将orm映射类写入缓存,减少重复创建类
    :param table_class: orm映射类
    :return:
    """
    logger.info('保存映射缓存: %s %s %s', table_name, ' => ', str(table_class))
    table_cache[table_name] = table_class


def get_cache(table_name: str) -> object:
    """
    根据表名获取orm映射类缓存
    :param table_name: 映射表名
    :return: orm映射类
    """
    logger.info('获取映射缓存: %s', table_name)
    return table_cache[table_name]


def search_class_by_name(table_name: str) -> bool:
    """
    根据表名查找缓存
    :param table_name: 映射表名
    :return: 是否命中缓存
    """
    logger.info('检查映射缓存: %s', table_name)
    return table_name in table_cache



import re

"""
功能特性类集合
"""

__all__ = ['WhereAble', 'ValueAble', 'SetAble', 'OrmObj', 'JoinAble', 'Parseable', 'Flushable', 'Pageable']


class OrmObj(object):
    """
    表映射基础类(标注作用)

    待添加:sql关键字保护策略算法
    """
    pass


class FieldsCallable(OrmObj):
    """
    可取出内置数据集合的功能类
    """

    def __init__(self):
        """
        初始化类功能实现的数据域
        """
        if not hasattr(self, '__field_callable_list'):
            setattr(self, '__field_callable_list', [])
            self.__field_callable_list = []

    def f_c_l_flush(self):
        """
        清洗内置数据集合
        :return:
        """
        setattr(self, '__field_callable_list', [])
        setattr(self, '_FieldsCallable__field_callable_list', [])

    def append(self, field_name: str):
        """
        内部方法:记录字段名
        :param field_name:
        :return:
        """
        f_list = getattr(self, '__field_callable_list')
        f_list.append(field_name)
        setattr(self, '__field_callable_list', f_list)
        self.__field_callable_list.append(field_name)

    def get_fields(self, f_list: list = []) -> dict:
        """
        获取字段映射
        :param list:
        :return:
        """
        cache = {}
        if hasattr(self, '__field_callable_list') and len(getattr(self, '__field_callable_list')) > 0:
            fs_name = getattr(self, '__field_callable_list')
        elif self.__field_callable_list:
            fs_name = self.__field_callable_list
        elif hasattr(self, '__default_arg_list'):
            fs_name = getattr(self, '__default_arg_list')
        else:
            fs_name = f_list

        for f_name in fs_name:
            if hasattr(self, f_name):
                if f_list:
                    if f_name in f_list:
                        cache[f_name] = getattr(self, f_name)
                        continue
                else:
                    cache[f_name] = getattr(self, f_name)
            else:
                print('警告:表(', getattr(self, 'table_map_key'), ')缺失字段(', f_name, '),表映射存在风险')
        return cache

    def get_field_list(self, f_list: list = []) -> list:
        """
        获取字段列表
        :param list:
        :return:
        """
        cache_list = list(self.get_fields(f_list=f_list).keys())
        cache = []
        for f_name in cache_list:
            cache.append(
                getattr(self, '__alias') + '.' +
                f_name
            )

        return cache

    def fields(self, fields: list = []):
        fields_list = self.get_field_list(fields)
        cache = re.sub('\[|\]|\\\'|\\\"', '', str(fields_list))
        return cache


class ValueAble(FieldsCallable):
    """
    可赋值化功能类
    """

    def __init__(self):
        """
        初始化类功能实现的数据域
        """
        FieldsCallable.__init__(self)

    def value_cause(self, args: dict) -> list:
        """
        将字典键值分离
        :param args: 字典对象
        :return:
        """
        keys = re.sub('\\\'', '', re.sub('\[', '(', re.sub('\]', ')', str(list(args.keys())))))
        values = re.sub('\[', '(', re.sub('\]', ')', str(list(args.values()))))
        return [
            keys
            ,
            values
        ]

    def values(self, fields: list = []) -> str:
        """
        获取条件赋值语句
        :param fields: 键值列表
        :return:
        """
        result = self.value_cause(self(fields))
        return result[0] + ' VALUES ' + result[1]


class WhereAble(FieldsCallable):
    """
    可条件化功能类
    """

    def __init__(self):
        """
        初始化类功能实现的数据域
        """
        FieldsCallable.__init__(self)

    def where_cause(self, args: dict, be_alias: bool = True) -> str:
        """
        将字典转换为sql条件语句
        :param args: 字典(条件字典)
        :return: sql条件语句
        """
        cache = []
        for key in args.keys():
            self_table_alias = \
                (getattr(self, '__alias') + '.'
                 if be_alias and getattr(self, '__alias') != getattr(self, 'table_map_key')
                 else '')
            cache.append(
                self_table_alias +
                str(key) +
                '=' +
                (str(
                    args[key]) if isinstance(args[key], int) or isinstance(args[key], float) else (
                        '{' + str(args[key]) + '}')))
        return re.sub('\{|\}', '\'', re.sub('\[|\]|\\\"|\\\'', '', re.sub(',', ' AND ', str(cache))))

    def where(self, fields: list = [], be_alias: bool = True) -> str:
        """
        获取条件执行语句
        :param fields:条件列表
        :return:
        """
        if len(fields) > 0 or (
                hasattr(self, '__field_callable_list') and len(getattr(self, '__field_callable_list')) > 0):
            args = getattr(self, 'get_fields')(fields)
        else:
            return ''
        return ' WHERE ' + getattr(self, 'where_cause')(args, be_alias)


class SetAble(WhereAble):
    """
    可更新化条件类
    """

    def __init__(self):
        """
        初始化类功能实现的数据域
        """
        FieldsCallable.__init__(self)

    def set(self, fields: list = []) -> str:
        """
        获取更新执行语句
        :param fields: 更新参数列表
        :return:
        """
        args = self.get_fields(fields)
        return ' SET ' + re.sub('AND', ',', self.where_cause(args))


class JoinAble(OrmObj):
    """
    可关联化功能类
    """

    def __init__(self):
        """
        初始化关联列表
        """
        if not hasattr(self, '__join_list'):
            setattr(self, '__join_list', [])
            self.__join_list = []

    def left_join(self, target, on_left_field: list, on_right_field: list):
        """
        左关联功能
        :param target: 关联实例
        :return: 自身实例
        """
        if not on_left_field or not on_right_field or len(on_left_field) > len(on_right_field):
            raise Exception('关联参数异常')
        if isinstance(target, JoinAble):
            setattr(self, '__join_list', [])
            getattr(self, '__join_list').append({
                'target': target,
                'left_field': on_left_field,
                'right_field': on_right_field
            })
            return self
        else:
            raise Exception('关联对象不支持!!!')

    def right_join(self, target, on_left_field: list, on_right_field: list):
        """
        右关联功能
        :param target: 关联实例
        :return: 自身实例
        """
        if isinstance(target, JoinAble):
            target.left_join(self, on_left_field, on_right_field)
            return self
        else:
            raise Exception('关联对象不支持!!!')

    def union(self):
        pass

    def exe_join(self) -> str:
        self_table_alias = \
            getattr(self, '__alias') if getattr(self, '__alias') != getattr(self, 'table_map_key') else ''

        result = [
            getattr(self, 'table_map_key') +
            (' AS ' if getattr(self, '__alias') != getattr(self, 'table_map_key') else '')
            + self_table_alias
        ]
        for join_obj in getattr(self, '__join_list'):
            # 获取关联对象
            obj = join_obj['target']
            join_obj_table_alias = \
                getattr(obj, '__alias') if getattr(obj, '__alias') != getattr(obj, 'table_map_key') else ''
            result.append(getattr(obj, 'table_map_key') +
                          (' AS ' if getattr(obj, '__alias') != getattr(obj, 'table_map_key') else '')
                          + join_obj_table_alias)
            # 获取关联键
            left_field = join_obj['left_field']
            right_field = join_obj['right_field']
            # 以关联左键为准
            on_fields_pair_sep = ' AND '
            on_fields_pair = []
            for index in range(len(left_field)):
                l_field = left_field[index]
                on_fields_pair.append(
                    self_table_alias + '.' + l_field +
                    ' = '
                    + join_obj_table_alias + '.' + right_field[index])
        return ' LEFT JOIN '.join(result) + ' ON ' + on_fields_pair_sep.join(on_fields_pair)


class Parseable(OrmObj):

    def read_from_dict(self, d: dict):
        """
        读取字典生成orm对象
        :param d:
        :return:
        """
        for key in d.keys():
            if hasattr(self, key):
                setattr(self, key, d[key])
            else:
                raise Exception('无法转换为' + str(getattr(self, 'table_map_key')) + '类型')
        return self

    def copy_to_obj(self, clz: OrmObj):
        res_obj = clz()
        for name in dir(self):
            if re.search('^_.*', name):
                continue
            if name not in dir(res_obj):
                raise Exception('无法复制的类型')


class Flushable(OrmObj):

    def flush(self):
        """
        清洗对象内部数据
        :return:
        """
        # 清洗参数列表缓存
        self.f_c_l_flush()
        for name in dir(self):
            if name.startswith('__') and name.endswith('__'):
                continue
            if name.startswith('_') \
                    and not name.startswith('__') \
                    and not re.search('[A-Z]', name) \
                    and getattr(self, name):
                setattr(self, name, 'none')


class Pageable(OrmObj):

    def limit(self, start: int, end: int):
        """
        写入上下标查询信息
        :param start: 记录起始位置
        :param end: 记录条数
        :return:
        """
        assert start > -1, '下标越界!'
        if end:
            assert end > -1, '上标越界!'
        setattr(self, '__limit_start', start)
        setattr(self, '__limit_end', end)
        setattr(self, '__execuable_limit_sql', f""" LIMIT {start}"""f""",{end}""" if end else '')
        return self

    def clear_limit_info(self):
        """
        清除上下标信息
        :return:
        """
        delattr(self, '__limit_start')
        delattr(self, '__limit_end')
        delattr(self, '__execuable_limit_sql')

    def page(self, page_num: int, page_size: int, pages_size: int = 10):
        """
        写入分页信息
        :param page_num:
        :param page_size:
        :return:
        """
        assert page_num > -1, '页码错误'
        assert page_size > 0, '页容错误'
        setattr(self, '__page_num', page_num)
        setattr(self, '__page_size', page_size)
        setattr(self, '__pages_size', pages_size)
        start = page_num * page_size
        end = page_size
        setattr(self, '__page_switch', True)
        return self.limit(start, end)

    def clear_page_info(self):
        """
        清除分页信息
        :return:
        """
        delattr(self, '__page_num')
        delattr(self, '__page_size')
        delattr(self, '__pages_size')
        setattr(self, '__page_switch', False)



from dophon_db.orm.manager_init import *
from dophon_db import properties

from dophon_logger import *

logger = get_logger(DOPHON)

logger.inject_logger(globals())


def init_orm(table_list=[], conn_kwargs={}):
    """
    初始化orm
    :return:
    """
    if hasattr(properties, 'db_cluster'):
        logger.info('分片数据库初始化')
        return ClusterManager()
    else:
        init_orm_manager(table_list, conn_kwargs)



# coding: utf-8
from dophon_db.mysql.binlog import ZipBinLog

from dophon_logger import *

logger = get_logger(DOPHON)

"""
xml文件增强功能封装单元
author:CallMeE
date:2018-06-01
"""

logger.inject_logger(globals())

class BinCache():
    _bin = ''
    _file = ''
    _false_fun = None

    def __init__(self, file):
        # 初始化原始bin对象
        self._bin = ZipBinLog.zip_as_bin(file)
        self._file = file

    def set_false_fun(self, fun):
        self._false_fun = fun

    def chk_diff(self):
        if self._bin == ZipBinLog.zip_as_bin(self._file):
            pass
        else:
            logger.info('文件发生增量更新(' + self._file + ')')
            # 执行增量更新方法
            if not self._false_fun:
                pass
            else:
                self._false_fun()
        # 重新初始化对象binlog信息
        self._bin = ZipBinLog.zip_as_bin(self._file)


def def_false_fun(file):
    logger.info('xml文件发生增量改变!(' + str(file) + ')')



# coding: utf-8
# 连接包装类
import pymysql
from dophon_db import properties

from dophon_logger import *

logger = get_logger(DOPHON)

"""
mysql连接实例(半成品)

待实现:;
1.配置文件配置连接属性
author:CallMeE
date:2018-06-01
"""

logger.inject_logger(globals())

namespace = [
    '_host',
    '_user',
    '_password',
    '_database',
    '_port',
    '_unix_socket',
    '_charset',
    '_sql_mode',
    '_read_default_file',
    '_conv',
    '_use_unicode',
    '_client_flag',
    '_cursorclass',
    '_init_command',
    '_connect_timeout',
    '_ssl',
    '_read_default_group',
    '_compress',
    '_named_pipe',
    '_no_delay',
    '_autocommit',
    '_db',
    '_passwd',
    '_local_infile',
    '_max_allowed_packet',
    '_defer_connect',
    '_auth_plugin_map',
    '_read_timeout',
    '_write_timeout',
    '_bind_address',
    '_binary_prefix'
]


class Connection(pymysql.connections.Connection):
    '''
    数据库连接参数默认为连接本地root账户

    目前只支持配置参数以及默认值:
    _host = 'localhost'
    _user = 'root'
    _password = 'root'
    _database = None
    '''
    _host = properties.pydc_host
    _user = properties.pydc_user
    _password = properties.pydc_password
    _database = properties.pydc_database
    _port = properties.pydc_port
    _charset = 'utf8'

    def __init__(self, **kwargs):
        __host = kwargs['host'] if 'host' in kwargs else (kwargs['__host'] if '__host' in kwargs else self._host)
        __port = kwargs['port'] if 'port' in kwargs else (kwargs['__port'] if '__port' in kwargs else self._port)
        __user = kwargs['user'] if 'user' in kwargs else (kwargs['__user'] if '__user' in kwargs else self._user)
        __password = kwargs['password'] if 'password' in kwargs else (
            kwargs['__password'] if '__password' in kwargs else self._password)
        __database = kwargs['database'] if 'database' in kwargs else (
            kwargs['__database'] if '__database' in kwargs else self._database)
        __charset = kwargs['charset'] if 'charset' in kwargs else (
            kwargs['__charset'] if '__charset' in kwargs else self._charset)
        super(Connection, self).__init__(host=__host, port=__port, user=__user, password=__password,
                                         database=__database,
                                         charset=__charset)

    def getConnect(self):
        return self

    def testConn(self):
        try:
            self.ping()
            return True
        except:
            return False

    def reConn(self):
        self.__init__(self._host, self._port, self._user, self._pwd, self._db, charset='utf8')



# coding: utf-8

import dophon_db.reader as reader
from dophon_db.mysql import PageHelper
from dophon_db import Connection
from dophon_db import Pool
from dophon_db import utils
from dophon_db.mysql.remote.Cell import Cell
import threading
import re
from dophon_db.mysql import binlog
from dophon_db.mysql.binlog import Schued
from dophon_db import properties
from dophon_db.mysql.sql_util import *
from dophon_db.const import *
from dophon_logger import *

logger = get_logger(DOPHON)
"""
后续开发;
(完成)1.远程读取mapperxml
(完成)2.语句对象定时更新暂定为本地xml文件增量binlog更新,后续开发远程增量更新(流程未定)
    (完成)2.1本地binlog处理对比
    (完成)2.2远程binlog处理对比
(半完成)3.binlog生成算法以及对比算法以及增量写入
(完成)4.细粒度事务控制
(完成)5.配置文件配置连接数
(完成)6.参照mybatis完成sql骨架拼写


单条语句执行demo:
# obj = getDbObj(project_path + '/mappers/ShopGoodsMapper.xml')
# setObjUpdateRound(obj, '2')
# obj.exe_sql("findGoodsList")

批量语句执行DEMO:
# obj=getDbObj(path=project_path +'/mysql/test.xml',debug=True)
# obj.exe_sql_obj_queue(queue_obj={"test":(1,2),"test":(2,3)})
或者
# obj.exe_sql_queue(method_queue=['test','test','test_s','test','test'],args_queue=[('1','2'),('2','3'),(),('3','4'),('3','4')])
"""

# 定义项目路径
project_path = properties.project_root

logger.inject_logger(globals())


class BlockingThreadError(Exception):
    pass


class curObj:
    _page = False
    _db = None
    _cursor = None
    _conn = None
    _debug = False
    _cursor = None
    sql = None
    lock = threading.Lock()

    def __init__(self, path: str, poolFlag: bool, debug: bool, db=None):
        """
        初始化结果集对象
        :param db: 连接工具实例(连接池或单个连接)
        :param path: 结果集管理xml路径
        :param poolFlag: 连接池标识
        :param debug: 调试标识
        """
        if not db:
            db = Pool.get_single_pool()
        self._debug = debug
        self._poolFlag = poolFlag
        self.sql = reader.Mapper()
        if poolFlag:
            # 连接池实例化
            self._pool = db
        else:
            self._db = db
        self.init_mapper(path)
        # global pool
        # pool = custom_pool

    def init_mapper(self, path):
        self.sql.openDom(path)
        self._path = path
        self._file_part_mark = re.sub(REGIXSTR.get_mapper_file_name, '', path.split('/')[len(path.split('/')) - 1])
        self._sqls = self.sql.getTree()[self._file_part_mark]

    def __getattr__(self, item):
        return CallableObject(self.exe_sql, item)

    def __setattr__(self, key, value):
        raise KeyError('unknown command')

    def get_tables(self):
        """
        获取自身关联数据库表数据
        :return:
        """
        self._cursor.execute("""
        SELECT table_name FROM information_schema.TABLES
        """)
        self._db.commit()
        result = utils.sort_result(self._cursor.fetchall(), self._cursor.description, [])
        self.__db_tables_list = result

    def refreash_sqls(self):
        """
        刷新sql语句(动态调试sql)
        ps:慎用
        :return:
        """
        self.init_mapper(self._path)

    def check_conn(self):
        """
        检查数据库连接
        :return:
        """
        try:
            if not self._db:
                # 无连接,需要获取连接
                if self._poolFlag:
                    # 连接池
                    self._conn = self._pool.getConn()
                    self._db = self._conn.getConnect()
                else:
                    # 单个连接理论上只执行一次,过后直接关闭
                    self._db = Connection()
        except Exception as e:
            logger.error(e)
            raise Exception('检查连接失败')

    def set_cursor(self):
        """
        # 初始化指针(如果不存在指针)

        :return:
        """
        if not self._cursor:
            self._cursor = self._db.cursor()
            # self.get_tables()

    def get_cursor(self):
        return self._cursor

    def get_sql(self, methodName: str, pageInfo, args=()):
        """
        # 获取sql语句(包含处理)

        :param methodName: 结果集代号
        :param pageInfo: 分页标识
        :param args: 结果集映射语句参数集
        :return:
        """
        # 单独连接实例化
        # 判断是否存在子节点
        if methodName not in self._sqls:
            logger.error('没有该方法!method: ' + str(methodName))
            raise Exception('没有该方法!method:' + str(methodName))
        _sql = self.sql.get_executable_sql(self._sqls, methodName, args)
        # 判断是否分页(总开关)
        if self._page:
            # 开启之后该实例所有语句都认为是 需要分页
            # 慎用!!!!
            # 分页
            _sql = _sql + 'limit ' + str((self._pageNum - 1) * self._pageSize) + ',' + str(self._pageSize)
        if pageInfo:
            # 分页
            _sql = _sql + PageHelper.depkg_page_info(pageInfo)
        # 判断是否骨架拼接
        if args:
            _sql = self.translate_sql_bond(_sql, args)
        # 去除注释与空格,换行等
        __sql = re.sub(REGIXSTR.get_at_least_one_blank, ' ', re.sub('<!--.*-->', ' ', _sql))
        return __sql

    def translate_sql_bond(self, _sql: str, args):
        """
        转义sql结果集骨架
        :param _sql: 结果集原始骨架
        :param args: 骨架参数集合
        :return: 转义后的sql语句
        """
        result_sql = _sql
        # 检查骨架实参传入类型,并作不同处理
        if type(args) is type(()):
            if re.match(REGIXSTR.mapper_bound_field_check, _sql):
                logger.error('骨架与参数不匹配')
                raise Exception('骨架与参数不匹配')
            result_sql = _sql % args[:]
        elif type(args) is type({}):
            for key in args.keys():
                reg_str = r'(\#|\$|\!|\@|\?)\{' + str(key) + '\}'
                if not re.search(reg_str, _sql):
                    '''
                    此处有几种情况:
                    1.语句骨架不存在该key的空位(多余参数)
                    2.骨架参数与骨架不对应(多余空位)
                    '''
                    pass
                else:
                    # 转义参数值,防止sql注入
                    e_value = escape(args[key])
                    # 转义骨架参数
                    result_sql = re.sub('\$\{' + str(key) + '\}', str(e_value), result_sql)
                    # 转义字符参数
                    result_sql = re.sub('\#\{' + str(key) + '\}', '\'' + str(e_value) + '\'', result_sql)
                    # 转义近似参数
                    # 左近似
                    result_sql = re.sub('\!\{' + str(key) + '\}', '\'%' + str(e_value) + '\'', result_sql)
                    # 右近似
                    result_sql = re.sub('\@\{' + str(key) + '\}', '\'' + str(e_value) + '%\'', result_sql)
                    # 全近似
                    result_sql = re.sub('\?\{' + str(key) + '\}', '\'%' + str(e_value) + '%\'', result_sql)
            # 多余空位检查
            if re.search('(\#|\$|\!|\@|\?)\{' + str(key) + '\}', result_sql):
                logger.error('存在无法配对的骨架参数')
                raise Exception('存在无法配对的骨架参数')
        else:
            try:
                result_sql = _sql % args[:]
            except Exception as e:
                logger.error(e + '\n')
                raise e
        return result_sql

    def set_page(self, pageNum: str, pageSize: str):
        """
        # 设定分页信息

        :param pageNum:页号(从0开始)
        :param pageSize: 页容(>0)
        :return:
        """
        self._pageNum = pageNum
        self._pageSize = pageSize
        self._page = True
        return self

    def initial_page(self):
        """
        重置结果集实例的分页信息
        :return:
        """
        self._page = False
        return self

    def exe_sql_obj_queue(self, queue_obj={}) -> dict:
        # 批量执行语句(整体版)
        """
        queue_obj中key为方法名,value为参数
        注意!!!!
        对于一个业务来说,一个sql方法只使用一次(因为有内部数据缓存)
        若其中有重复方法,建议用分割版
        """

        if queue_obj:
            methods = list(queue_obj.keys())
            args = list(queue_obj.values())
            return self.exe_sql_queue(method_queue=methods, args_queue=args)
        else:
            logger.error('queue_obj参数不正确')
            raise Exception('queue_obj参数不正确')

    def exe_sql_queue(self, method_queue=[], args_queue=[]) -> dict:
        # 批量执行语句(拆分版)
        """
        method_queue中存放顺序执行的sql方法名[str]
        args_queue中存放对应下标方法的参数元组[()]
        若其中包含select无条件参数语句,请用空元组()占位
        """
        self.lock.acquire(blocking=True)
        result = {}
        # 参数检查
        if not method_queue:
            logger.error('语句方法为空')
            raise Exception('语句方法为空')
            return
        if not args_queue:
            logger.error('语句参数列表为空')
            raise Exception('语句参数列表为空')
            return
        self.check_conn()
        self.set_cursor()
        try:
            # 开启事务
            # 批量取语句(以方法名为准,多于参数队列元素将丢弃)
            while method_queue:
                method = method_queue.pop(0)
                args = args_queue.pop(0)
                """
                对于增改查来说,并不需要分页,参数列表是必须的
                """
                _sql = self.get_sql(methodName=method, args=args, pageInfo=None)
                # 执行sql语句
                exc_result = self._cursor.execute(_sql)
                '''
                尝试执行语句成功后会解析结果集
                '''
                if re.match(REGIXSTR.select_sql_check, _sql):
                    data = self._cursor.fetchall()
                    description = self._cursor.description
                else:
                    data = [[self._cursor.rowcount]]
                    description = [['row_count']]
                current_result = utils.sort_result(data, description, [])
                # 调试模式打印语句
                if self._debug:
                    print_debug(methodName=method, args=args, sql=_sql, result=self._cursor.rowcount)
                    # 事务提交(pymysql要求除查询外所有语句必须手动提交)
                if method in result:
                    result[method + str(len(result))] = current_result
                else:
                    result[method] = current_result
        except Exception as e:
            logger.error('SQL错误: %s , 语句为: %s' % (str(e), _sql))
            self._db.rollback()
            logger.error('事务回滚' + str(method_queue))
            raise e
        else:
            self._db.commit()
            logger.info('事务提交' + str(method_queue))
        finally:
            self.lock.release()
            # 关闭连接
            self.close()
            return result

    def exe_sql(self, methodName='', pageInfo=None, args=()) -> list:
        """
        # 执行单条语句,返回结果列表(select more)

        # 防报错参数设定默认值
        :param methodName: 结果集映射代号
        :param pageInfo: 结果集分页信息
        :param args: 结果集查询参数
        :return: <select>  ==>  查询结果
                  <insert,update,delete>  ==>  有效行数
        """
        self.lock.acquire(blocking=True)
        # 参数检查
        if not re.sub('\s+', '', methodName):
            logger.error('语句方法为空')
            raise Exception('语句方法为空')
            return
        self.check_conn()
        self.set_cursor()
        # 定义返回结果集
        result = []
        try:
            if pageInfo and type(pageInfo) is type({}):
                _sql = self.get_sql(methodName=methodName, pageInfo=pageInfo, args=args)
            else:
                _sql = self.get_sql(methodName=methodName, args=args, pageInfo=None)
        except Exception as ex:
            logger.error(ex)
            return result
        try:
            # 试执行语句
            self._cursor.execute(_sql)
            self._db.commit()
            self.initial_page()
            '''
            尝试执行语句成功后会解析结果集
            '''
            if re.match(REGIXSTR.select_sql_check, _sql):
                data = self._cursor.fetchall()
                description = self._cursor.description
            else:
                data = [[self._cursor.rowcount]]
                description = [['row_count']]
            result = utils.sort_result(data, description, result)
        except Exception as e:
            self._db.rollback()
            self.initial_page()
            logger.error("执行出错,错误信息为:" + str(e) + 'sql语句为:' + _sql)
        finally:
            # 关闭连接
            self.close()
            # 调试模式语句执行信息打印
            if self._debug:
                print_debug(methodName=methodName, args=args, sql=_sql, result=result)
            self.lock.release()
        # 非查询语句返回影响行数
        if result:
            if re.match(REGIXSTR.select_sql_check, _sql):
                return result
            else:
                return data[0][0]
        else:
            return []

    def exe_sql_single(self, methodName='', pageInfo=None, args=()) -> object:
        """
        返回单个对象(select one)
        # 防报错参数设定默认值
        :param methodName: 结果集映射代号
        :param pageInfo: 结果集分页信息
        :param args: 结果集查询参数
        :return: <select>  ==>  查询结果
                  <insert,update,delete>  ==>  有效行数
        """
        result = self.exe_sql(methodName=methodName, pageInfo=pageInfo, args=args)
        if isinstance(result, list):
            # 列表类型执行取值
            if 1 < len(result):
                # 多个结果
                logger.error('过多结果')
                raise Exception('过多结果')
            elif 0 == len(result):
                return None
            else:
                return result[0]
        else:
            return result

    def close(self):
        """
        (单个连接):关闭数据库连接
        (连接池):归还连接
        :return:
        """
        # print(id(self._pool))
        self._cursor.close()
        if self._poolFlag:
            # 为连接池定义
            self._pool.closeConn(self._conn)
            # 归还连接后清除指针
            self._cursor = None
            self._db = None
            self._conn = None
        else:
            # 为单独连接定义
            self._db.close()

    def insert_to_update_dispacther(self, millionSecond):
        """
        # 定义插入更新调度方法

        :param millionSecond: 更新频率
        :return:
        """
        if isinstance(millionSecond, int):
            w_time = millionSecond
            pass
        else:
            try:
                w_time = int(millionSecond)
            except Exception as e:
                logger.error(e)

        # 此处为增量更新代码
        '''
        临时思路
        1.设定定时间隔
        2.传入当前语句对象
        3.内部压缩保存binlog
        4.定时完毕重新获取语句,获取新语句对象binlog
        5.对比binlog
            5.1若更新后binog无差异则不作处理
            5.2若存在差异,替换语句对象
        '''
        self._bin_cache = binlog.BinCache(self._path)
        # 添加变更处理
        self._bin_cache.set_false_fun(self.refreash_sqls)
        # 调度器添加任务
        Schued.sech_obj(fun=self._bin_cache.chk_diff, delay=w_time).enter()

    """
    细粒度sql语句执行组
    """

    def execute(self, method_name: str, pageInfo=None, args=()):
        if self.lock.locked():
            raise BlockingThreadError('还有事务尚未提交!!!')
        self.lock.acquire(blocking=True)
        self._cursor.execute(sql)

    def commit(self):
        self._db.commit()
        self.lock.release()

    def rollback(self):
        self._db.rollback()
        self.lock.release()


def c_prop(clz: type, prop_name: str, prop_value=None, use_setter: bool = True, use_getter: bool = True):
    """
    生成类内属性方法
    :return:
    """
    in_setter = None
    in_getter = None
    setattr(
        clz,
        '_' + prop_name,
        prop_value,
    )
    if use_setter:
        in_setter = c_prop_setter(prop_name)
    if use_getter:
        in_getter = c_prop_getter(prop_name)
    setattr(
        clz,
        prop_name,
        property(in_getter, in_setter)
    )

    return clz


def c_prop_setter(prop_name: str) -> classmethod:
    """
    生成setter方法
    :return:
    """
    setter_template = compile(
        'def setter_' + prop_name + '(self,value):' +
        '\n\tself._' + prop_name + ' = value' +
        '',
        'exec'
    )
    setter_function_code = [c for c in setter_template.co_consts if isinstance(c, types.CodeType)][0]
    setter_method = types.FunctionType(setter_function_code, {})
    return setter_method


def c_prop_getter(prop_name: str) -> classmethod:
    """
    生成getter方法
    :return:
    """
    getter_template = compile(
        'def getter_' + prop_name + '(self):' +
        '\n\treturn self._' + prop_name,
        '',
        'exec'
    )
    getter_function_code = [c for c in getter_template.co_consts if isinstance(c, types.CodeType)][0]
    getter_method = types.FunctionType(getter_function_code, {})
    return getter_method


def setObjUpdateRound(obj: curObj, second: int):
    """
    设置结果集映射实例定时更新
    :param obj: 结果集映射实例
    :param second: 更新频率
    :return:
    """
    if isinstance(obj, curObj):
        obj.insert_to_update_dispacther(second)
    else:
        logger.error('类型错误!!!!')
        raise Exception('类型错误!!!!')


def print_debug(methodName: str, sql: str, args: dict, result: list):
    """
    # 调试模式下的语句信息打印

    :param methodName: 结果集映射实例的结果集代号
    :param sql: 结果集生成语句
    :param args: 结果集实例执行参数
    :param result: 结果集实例执行结果
    :return:
    """
    print('METHOD:==>' + methodName)
    print('SQL:=====>' + sql)
    print('PARAMS:==>' + str(args))
    if isinstance(result, list):
        if result and result[0]:
            # 拿出列名
            print('ROWS:====>' + str(list(result[0].keys())))
            print('RESULT:==>' + str(list(result[0].values())))
            for r in result[1:]:
                print('=========>' + str(list(r.values())))
        else:
            print('ROWS:====>None')
            print('RESULT:==>None')
    if isinstance(result, int):
        print('ROWS:====>', result)


def whereCause(args: dict) -> str:
    """
    将字典转换为sql条件语句
    :param args: 字典(条件字典)
    :return: sql条件语句
    """
    cache = []
    for key in args.keys():
        cache.append(str(key) + '=' + str(args[key]))
    return 'WHERE ' + re.sub('\[|\]|\\\"|\\\'', '', re.sub(',', ' AND ', str(cache)))


update_round = setObjUpdateRound
where = whereCause

__all__ = [
    'update_round', 'where', 'CastableObject'
]

# 检测是否开启sql热更新模式
if properties.pydc_xmlupdate_sech:
    Schued.start_sech()


class CallableObject:
    """
    可执行载体类
    """

    def __init__(self, callable_entity, execution_body: str):
        assert callable(callable_entity), 'unknown entity'
        self.__callable = callable_entity
        self.__execution = execution_body

    def __call__(self, *args, **kwargs):
        return self.__callable(methodName=self.__execution, pageInfo=None, args=kwargs)



from . import db_init


class SqliteManager():
    pass


singleton = SqliteManager()


def init():
    db_init.init_config()
    for k, v in db_init.cache_config.items():
        setattr(singleton, k.replace('sqlite_', ''), v)
    return singleton



DICT = dict
OBJECT = object

# 定义返回类型(默认字典类型)
result_type = DICT


def get_db_cluster_info():
    from dophon_db import properties
    result = []
    for cluster in properties.db_cluster:
        alias = cluster.get('alias', 'default_db')
        host = cluster.get('host', 'localhost')
        port = cluster.get('port', 3306)
        db = cluster.get('database', 'database')
        user = cluster.get('user', 'root')
        password = cluster.get('password', 'root')
        chartset = cluster.get('chartset', 'utf8')
        tables = cluster.get('tables', []) if isinstance(cluster.get('tables'), list) else \
            [cluster.get('tables')] if 'tables' in cluster else []
        result.append({
            alias: {
                'alias': alias,
                '__host': host,
                '__port': port,
                '__database': db,
                '__user': user,
                '__password': password,
                '__chartset': chartset,
                'table_list': tables
            }
        })
    return result


def sort_result(data: list, description: tuple, result: list) -> list:
    """
    # 整理结果集并返回
    :param data: 数据集
    :param description:数据描述
    :param result: 结果列表(或许产生多个集合)
    :return:
    """
    for index in range(len(data)):
        item = data[index]
        r_item = {}
        if result_type is DICT:
            for i in range(len(item)):
                colName = description[i][0]
                val = item[i]
                value = 'none'
                if type(val) is not type(None):
                    value = str(val)
                # 组装data
                r_item[colName] = value
            # 组装结果集
            result.append(r_item)
        elif result_type is OBJECT:
            class_obj = type(str(id(data)), (), {})
            # 组装返回临时类
            for i in range(len(item)):
                colName = description[i][0]
                val = item[i]
                value = None
                if type(val) is not type(None):
                    value = str(val)
                class_obj = c_prop(class_obj, colName, value, use_setter=False)
            result.append(class_obj())
    return result



import sqlite3
import types
from dophon_db import properties
from dophon_logger import *

logger = get_logger(DOPHON)

logger.inject_logger(globals())

config = properties.sqlite

cache_config = {}


class Table():
    """
    数据库表类
    """

    def __init__(self, table_structure: set):
        for field in table_structure:
            setattr(self,f'_{field}',None)
            # 获取指定列名数据(property -> get)
            getter_module_code = compile(
                f'@property\ndef {field}(self):\n\treturn self._{field}',
                '',
                'exec'
            )
            gfunction_code = [c for c in getter_module_code.co_consts if isinstance(c, types.CodeType)][0]
            getter_method = types.FunctionType(gfunction_code, {})
            setattr(self, f'{field}', getter_method)
            # 获取指定列名数据(property -> set)
            setter_module_code = compile(
                f'@{field}.setter\ndef set_{field}(self,val):\n\tprint("不可修改的参数")',
                '',
                'exec'
            )
            sfunction_code = [c for c in setter_module_code.co_consts if isinstance(c, types.CodeType)][0]
            setter_method = types.FunctionType(sfunction_code, {})
            setattr(self, f'set_{field}', setter_method)
            setattr(self, field, property(getter_method, setter_method))
        self.__fields = table_structure


class DBObj():
    """
    数据库对象
    """
    __database = ':memory:'

    def __init__(self, database: str = ''):
        self.__database = database if database else self.__database
        # 查询所有的表
        result = self.execute('select name from sqlite_master where type=\'table\' order by name;')
        self.table_fields_struct = {}
        for item in result['data']:
            table_name = item[0]
            # 查询所有表结构
            inner_result = self.execute(f'select * from {table_name};')
            table_fields = set({})
            for table_column in inner_result['desc']:
                table_fields.add(table_column[0])
            self.table_fields_struct[table_name] = table_fields
            setattr(self, f'{table_name}', Table(table_fields))

    def execute(self, sql: str) -> dict:
        """
        执行语句,整理结果集
        :param sql: sql语句
        :return:
        """
        init_conn = sqlite3.connect(self.__database)
        init_cursor = init_conn.cursor()
        init_cursor.execute(sql)
        result = {
            'desc': init_cursor.description,
            'data': init_cursor.fetchall()
        }
        init_conn.commit()
        init_conn.close()
        return result

    def sort_result(self, result: dict):
        """
        整理结果集
        :param result:
        :return:
        """
        dict_result = []
        for result_item in result['data']:
            cache = {result['desc'][index][0]: result_item[index] for index in range(len(result_item))}
            dict_result.append(cache)
        return dict_result


def init_config():
    """
    初始化sqlite配置
    :return:
    """
    if cache_config:
        logger.warning('未生效任何配置')
        return
    for alias_name, alias_config in config.items():
        if isinstance(alias_config, dict):
            cache_config['sqlite_' + alias_name] = DBObj(alias_config['database'])



# coding: utf-8
from dophon_db.mysql.remote import Cell

"""
远程xml模块工厂
author:CallMeE
date:2018-06-01


# 开启远程xml推荐导入本模块获取实例
# import mysql.remote as remote
"""


# 工厂模式获取远程细胞实例
def getCell(file_name, remote_path, read_only=False):
    return Cell.get_cell(file_name=file_name, remote_path=remote_path, read_only=read_only)


from datetime import datetime
import sys
import re
import struct

from dophon_logger import *

logger = get_logger(DOPHON)
logger.inject_logger(globals())

db_type_python_dict = {
    # 字符串类型
    'char': {
        'type': str,
        'str_name': 'str',
        'min_length': 0,
        'max_length': 255
    },
    'varchar': {
        'type': str,
        'str_name': 'str',
        'min_length': 0,
        'max_length': 65535
    },
    'tinytext': {
        'type': str,
        'str_name': 'str',
        'min_length': 0,
        'max_length': 255
    },
    'text': {
        'type': str,
        'str_name': 'str',
        'min_length': 0,
        'max_length': 65535
    },
    'mediumtext': {
        'type': str,
        'str_name': 'str',
        'min_length': 0,
        'max_length': 16777215
    },
    'longtext': {
        'type': str,
        'str_name': 'str',
        'min_length': 0,
        'max_length': 4294967295
    },
    'enum': {
        'type': tuple,
        'str_name': 'tuple',
        'min_length': 0,
        'max_length': 65535 * 2
    },
    'set': {
        'type': set,
        'str_name': 'set',
        'min_length': 0,
        'max_length': 64 * 8
    },
    # 日期类型
    'date': {
        # 'type': datetime,
        'type': str,
        'str_name': 'datetime',
        'min_length': 0,
        'max_length': 65535
    },
    'time': {
        'type': datetime,
        'str_name': 'datetime',
        'min_length': 0,
        'max_length': 65535
    },
    'year': {
        'type': datetime,
        'str_name': 'datetime',
        'min_length': 0,
        'max_length': 65535
    },
    'datetime': {
        'type': datetime,
        'str_name': 'datetime',
        'min_length': 0,
        'max_length': 65535
    },
    'timestemp': {
        'type': datetime,
        'str_name': 'datetime',
        'min_length': 0,
        'max_length': 65535
    },
    # 浮点类型
    'float': {
        'type': float,
        'str_name': 'float',
        'min_length': 0,
        'max_length': sys.maxsize
    },
    'double': {
        'type': float,
        'str_name': 'float',
        'min_length': 0,
        'max_length': sys.maxsize
    },
    'tinyint': {
        'type': int,
        'str_name': 'int',
        'min_length': 0,
        'max_length': 1
    },
    'smallint': {
        'type': int,
        'str_name': 'int',
        'min_length': 0,
        'max_length': 2
    },
    'mediumint': {
        'type': int,
        'str_name': 'int',
        'min_length': 0,
        'max_length': 3
    },
    # 整数类型
    'int': {
        'type': int,
        'str_name': 'int',
        'min_length': 0,
        'max_length': 4
    },
    'bigint': {
        'type': int,
        'str_name': 'int',
        'min_length': 0,
        'max_length': 8
    }
}


# 计算数字字节数
def count_int_bytes(num: int):
    '''
    计算整形所占内存字节数
    :param num:
    :return:
    '''
    byte_num = int(len(re.sub('0b', '', bin(num))) / 8) + 1
    return byte_num


def count_str_bytes(string: str):
    '''
    计算字符串所占内存字节数
    :param string:
    :return:
    '''
    # byte_num = int(len(re.sub('0b', '', bin(int(string.encode().hex(), 16))))) + 1
    byte_num = len(string)
    return byte_num


def count_float_bytes(float_num: float):
    '''
    暂时弃用
    :param float_num:
    :return:
    '''
    byte_num = int(len(re.sub('0b', '', bin(int(hex(struct.pack("<f", float_num)), 16))))) + 1
    return byte_num


def check_data(data, type_str: str):
    '''
    检查数据是否合法
    :param data: 数据
    :param type_str: 类型字符串
    :return:
    '''
    for key in db_type_python_dict.keys():
        if key == re.sub('\(.*\)', '', type_str):
            data_struct_info = db_type_python_dict[key]
            max_length = data_struct_info['max_length']
            min_length = data_struct_info['min_length']
            length_info = re.sub('\(|\)', '', re.sub(key, '', type_str))
            if len(length_info) > 0:
                try:
                    max_length = int(length_info)
                except Exception as e:
                    logger.error(e)
            try:
                data_bytes = None

                if data_struct_info['type'] is int and isinstance(data, int):
                    data_bytes = count_int_bytes(data)
                elif data_struct_info['type'] is str and isinstance(data, str):
                    data_bytes = count_str_bytes(data)
                elif data_struct_info['type'] is datetime and isinstance(data, datetime):
                    data_bytes = count_str_bytes(data.strftime('yyyy-MM-dd HH:mm:ss'))

                if data_bytes and data_bytes > min_length - 1 and data_bytes < max_length + 1:
                    return True
                else:
                    return False
            except:
                err_msg = '数据类型错误( data_type = ' + \
                          str(type(data)) + \
                          ' , db_type = ' + \
                          type_str + \
                          ' , required_type = ' + \
                          str(data_struct_info['type']) + ' ) '
                logger.error(err_msg)
                raise Exception(err_msg)
    err_msg = '不支持的数据类型( data = ' + \
              (type(data)) + \
              ' ) '
    logger.error(err_msg)
    raise Exception(
        err_msg
    )


def set_check(data_type):
    def fun(f):
        def arg(*args, **kwargs):
            value = args[1] if len(args) > 1 else kwargs['value']
            if check_data(value, data_type):
                # 数据类型校验通过
                pass
            else:
                try:
                    # 尝试强制转换
                    db_type_python_dict[re.sub('([^a-zA-Z])','',data_type)]['type'](
                        value
                    )
                except:
                    err_msg = '数据类型校验不通过( data = ' + \
                              value + \
                              ' , data_type = ' + \
                              str(type(value)) + \
                              ' , db_type = ' + \
                              data_type + \
                              ' )'
                    logger.error(err_msg)
                    raise Exception(
                        err_msg
                    )
            result = f(*args, **kwargs)
            return result

        return arg

    return fun



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['dophon_db',
 'dophon_db.const',
 'dophon_db.mysql',
 'dophon_db.mysql.binlog',
 'dophon_db.mysql.remote',
 'dophon_db.mysql.sql_util',
 'dophon_db.orm',
 'dophon_db.orm.db_obj',
 'dophon_db.orm.db_obj.function_class',
 'dophon_db.reader',
 'dophon_db.sqllite',
 'dophon_db.utils']

package_data = \
{'': ['*']}

install_requires = \
['PyMySQL', 'dophon-logger']

setup_kwargs = {
    'name': 'dophon-db',
    'version': '1.2.6a3',
    'description': 'dophon mysql module',
    'long_description': None,
    'author': 'CallMeE',
    'author_email': 'ealohu@163.com',
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
